// Adding new story
"use strict";
// globals: window, document, DH, Android, setTimeout

var ST = ST || {};

ST.onAddClick = function () {
    // add new story
    var name = document.getElementById('name'),
        s = name.value.trim();

    // name must be non-empty
    if (s === '') {
        name.focus();
        return;
    }

    // extra length control (gboard in android allows more than maxlength)
    if (s.length >= 50) {
        ST.error('Story title cannot be longer than 50 characters!');
        return;
    }

    // clear cache
    DH.storage.erase('ST.cache.stories');
    DH.storage.erase('ST.cache.storiesTime');

    // add new story
    ST.request('add.php', 'name=' + encodeURIComponent(s),
        function (aResponse) {
            // success
            console.log('received', aResponse.code, aResponse.message, aResponse.detail);
            // add new story to favorites
            if (aResponse.code > 0) {
                ST.favorite.add(aResponse.code, s, 0);
            }
            // open page
            DH.storage.writeNumber('ST.story.code', aResponse.code);
            Android.loadUrl('file:///android_asset/story.html');
        });
};

ST.onNameKeyPress = function (event) {
    // enter will submit name
    if (event.key === 'Enter') {
        ST.onAddClick(event);
        event.preventDefault();
    }
};

ST.onBack = function () {
    Android.loadUrl('file:///android_asset/android.html');
};

window.addEventListener('DOMContentLoaded', function () {
    // initialize window
    DH.console.showOnMultipleClicks(true);
    DH.metrics.init('story', ST.appVersion || 0, undefined, undefined, true);
    //DH.metrics.log('add');
    DH.metrics.errorLogger(10);
    // callbacks
    document.getElementById('add').addEventListener('click', ST.onAddClick);
    document.getElementById('name').addEventListener('keypress', ST.onNameKeyPress, true);

    /*
    document.getElementById('test').addEventListener('click', function () {
        try {
            DH.storage.writeNumber('ST.story.code', 3);
            Android.loadUrl('file:///android_asset/story.html');
        } catch (e) {
            alert(e);
        }
    }, true);
    */

    // show keyboard on android
    if (typeof Android === 'object') {
        setTimeout(function () {
            Android.keyboardImplicit();
        }, 500);
    }
});
