// Main page, list of all stories
"use strict";
// globals: DH, window, document, setTimeout, Android, Billing

var ST = ST || {};

ST.startTime = Date.now();

ST.page = DH.storage.readNumber('ST.page', 1);

ST.onBack = function () {
    Android.loadUrl('file:///android_asset/android.html');
};

ST.onClickStory = function (event) {
    // click on story will read it
    console.log('Reading story.html with code ' + event.target.dataStoryId);
    DH.storage.writeNumber('ST.story.code', event.target.dataStoryId);
    Android.loadUrl('file:///android_asset/story.html');
};

ST.plainSummary = function (aSentences) {
    // return sentences joined together except special commands like #p or #br
    var i, s = [];
    for (i = 0; i < aSentences.length; i++) {
        if (['#p', '#br'].indexOf(aSentences[i]) < 0) {
            s.push(aSentences[i]);
        }
    }
    s = s.join(' ');
    // append 2 or 3 dots
    if (s.substr(-1, 1) === '.') {
        s += '..';
    } else {
        s += '...';
    }
    return s;
};

ST.renderStories = function (aResponse, aParent, aScroll) {
    // render stories
    console.log('ST.renderStories start ' + (Date.now() - ST.startTime));
    var i, story, h2, p, div, views, storybuttons, read, stories = aParent || document.getElementById('stories'), seen,
        instructions_stories = document.getElementById('instructions_stories'),
        recent = DH.storage.readNumber('ST.story.code', 0),
        scroll = DH.storage.readBoolean('ST.story.scroll', true);
    console.log('scroll', scroll);
    stories.textContent = '';

    function showRecentH2() {
        if (aScroll || scroll) {
            if (ST.recentH2.scrollIntoViewIfNeeded) {
                ST.recentH2.scrollIntoViewIfNeeded();
            } else {
                ST.recentH2.scrollIntoView();
            }
        }
    }

    // render all stories
    console.log(aResponse.detail.length, 'stories loaded');
    for (i = 0; i < aResponse.detail.length; i++) {
        story = aResponse.detail[i];
        seen = DH.storage.readNumber('ST_' + story.id, -1) + 1;
        //console.log('Story', story.id, story.name, story.count, seen);
        // wrapping div
        div = document.createElement('div');
        div.dataStoryId = story.id;
        //div.addEventListener('click', ST.onClickStory);
        if (story.id === 1) {
            instructions_stories.appendChild(div);
        } else {
            stories.appendChild(div);
        }
        // views and likes (currently not implemented)
        views = document.createElement('span');
        views.className = 'views';
        views.textContent = '65👁 16♥';
        //div.appendChild(views);
        // title
        h2 = document.createElement('h2');
        h2.textContent = story.name;
        h2.dataStoryId = story.id;
        div.appendChild(h2);
        // summary
        p = document.createElement('p');
        p.id = 'summary' + story.id;
        p.className = 'summary';
        p.textContent = ST.plainSummary(story.summary);
        p.dataStoryId = story.id;
        div.appendChild(p);
        // storynav
        storybuttons = document.createElement('div');
        storybuttons.className = 'storybuttons';
        p.appendChild(storybuttons);
        // read
        read = document.createElement('button');
        read.className = 'read';
        read.textContent = 'read more';
        read.dataStoryId = story.id;
        read.addEventListener('click', ST.onClickStory);
        // console.log('Seen', seen, 'total', story.count);
        if (seen < story.count) {
            read.textContent = 'read more, ' + (story.count - seen) + ' new';
            read.classList.add('new');
        }
        read.dataStoryId = story.id;
        //p.appendChild(read);
        storybuttons.appendChild(read);
        // scroll to recent
        if (story.id === recent) {
            ST.recentH2 = h2;
            setTimeout(showRecentH2, 300);
        }
    }
    // show "Start new story" only when loading is finished
    document.getElementById('start_new_story').style.display = 'initial';
    if (scroll) {
        DH.storage.erase('ST.story.scroll');
    }
    console.log('ST.renderStories end ' + (Date.now() - ST.startTime));
};

ST.renderSection = function (aData, aSectionId, aScroll, aScrollToPager) {
    // render section
    //console.log('ST.renderSection', aSectionId);
    //console.log('ST.renderSection ' + aSectionId + ' start ' + (Date.now() - ST.startTime));
    var section = document.getElementById(aSectionId), obj;
    try {
        // empty response
        if (!aData) {
            console.error('empty data');
            section.classList.add('error');
            section.textContent = 'Server is busy, try again later';
            return;
        }
        // convert to json
        obj = JSON.parse(aData);
        //console.log('obj', obj);
        // received data but it's not object
        if (!obj) {
            console.error('no obj ' + aData);
            section.classList.add('error');
            section.textContent = 'Server is busy, try again later';
            return;
        }
        // no stories in detail
        if (!obj.detail) {
            section.classList.remove('error');
            section.textContent = obj.message;
            return;
        }
        // error code
        if (obj.code !== 0) {
            console.error(obj.code + ' ' + obj.message);
            section.classList.add('error');
            section.textContent = obj.message;
        }
        // render stories
        section.classList.remove('error');
        ST.renderStories(obj, section, aScroll);
        //console.log('ST.renderSection ' + aSectionId + ' end ' + (Date.now() - ST.startTime));
        if (aScrollToPager) {
            document.getElementById('pagestop').scrollIntoView();
        }
        return obj;
    } catch (e) {
        console.error(e);
        section.classList.add('error');
        section.textContent = 'Server error, try again later';
    }
};

ST.renderPromotedStories = function (aData) {
    // render promoted stories
    ST.renderSection(aData, 'promoted_stories');
};

ST.onClickPage = function (event) {
    ST.page = parseInt(event.target.textContent, 10);
    DH.storage.writeNumber('ST.page', ST.page);
    event.target.style.color = 'black';
    ST.silentRequest('stories-section.php', 'section=page&page=' + encodeURIComponent(ST.page), function (aData) {
        ST.renderAllStories(aData, true);
    });
};

ST.renderPages = function (aParentId, aObject) {
    // render pagination
    var pages = document.getElementById(aParentId), i, a, max;
    pages.textContent = '';
    if (aObject && aObject.extra && aObject.extra.total) {
        console.log('o = ', aObject);
        max = Math.ceil(aObject.extra.total / aObject.extra.pageSize) + 1;
        for (i = 1; i < max; i++) {
            a = document.createElement('span');
            if (i === ST.page) {
                a.style.color = 'navy';
            } else {
                a.style.color = 'green';
            }
            a.textContent = i < max - 1 ? i + ',' : i;
            a.onclick = ST.onClickPage;
            pages.appendChild(a);
            pages.appendChild(document.createTextNode(' '));
        }
    }
};

ST.renderAllStories = function (aData, aScrollToPager) {
    // render all stories (paginated)
    var o = ST.renderSection(aData, 'all_stories', false, aScrollToPager);
    ST.renderPages('pages', o);
    ST.renderPages('pagestop', o);
};

ST.onStartNewStory = function () {
    // start new story
    if (!window.hasOwnProperty('Billing') || !Billing.isAvailable()) {
        DH.splash('Error', 'OK', 'pink', 'Billing service is not available, wait please...');
        return;
    }
    if (!Billing.purchaseExists('story_limit')) {
        var s = DH.splash('Wait', ['Buy', 'Cancel'], 'skyblue', 'To contribute new story or write sentences you need to make a one-time purchase of $1', function (aButton) {
            if (aButton === 'Buy') {
                Billing.purchase('story_limit', function (aOk, aSku) {
                    console.log('purchased', aOk, aSku);
                    if (aOk) {
                        Android.loadUrl('file:///android_asset/add.html');
                    }
                });
            }
        }, '80vw');
        s.bgClickDisable();
        return;
    }
    Android.loadUrl('file:///android_asset/add.html');
};

ST.onModerate = function () {
    // open moderation window
    Android.loadUrl('file:///android_asset/moderate.html');
};

ST.onAdventure = function () {
    // Show adventure for this month
    DH.storage.writeNumber('ST.story.code', 1);
    Android.setShared('1');
    Android.loadUrl('file:///android_asset/adventure/adventure.html');
};

window.addEventListener('DOMContentLoaded', function () {
    console.log('start ' + (Date.now() - ST.startTime));
    // initialize window
    DH.console.showOnMultipleClicks(true);
    // metrics
    DH.metrics.init('story', ST.appVersion || 0, undefined, undefined, true);
    DH.metrics.log('stories');
    DH.metrics.errorLogger(10);
    // callbacks
    document.getElementById('start_new_story').addEventListener('click', ST.onStartNewStory);
    document.getElementById('moderate').addEventListener('click', ST.onModerate);
    // show moderation button only for moderators
    if (ST.passMod) {
        document.getElementById('moderate').style.display = '';
    }
    // show stories
    ST.silentRequest('stories-section.php', 'section=promoted', ST.renderPromotedStories);
    ST.silentRequest('stories-section.php', 'section=page&page=' + encodeURIComponent(ST.page), ST.renderAllStories);
    // show favorites
    ST.favorite.showNewCount();
    document.getElementById('envelope').onclick = ST.favorite.showNewStories;
    // adventure
    document.getElementById('adventure').onclick = ST.onAdventure;
    // show new rules
    /*
    if (!DH.storage.readBoolean('ST.rules.signed', false)) {
        setTimeout(function () {
            document.getElementById('rules').style.display = 'block';
            document.getElementById('closerules').onclick = function () {
                document.getElementById('rules').style.display = 'none';
                DH.storage.writeBoolean('ST.rules.signed', true);
            };
        }, 1000);
    }
    */
    if (ST.server !== 'https://ghost.sk/story/') {
        document.getElementById('title').textContent = '(Story)';
    }
    console.log('Normal start');
    console.log('start done ' + (Date.now() - ST.startTime));

    document.getElementById('test').onclick = function () {
        Billing.test(['story_adopt', 'story_promote', 'story_limit']);
    };
    document.getElementById('test2').onclick = function () {
        DH.console.show(true);
    };
});

