// Billing functions and callbacks
"use strict";
// globals: document, window, DH, Billing

var ST = window.ST || {};

Billing.onPurchase = function (aSku) {
    // Callback when purchase is made
    Billing.log('onPurchase ' + aSku);
    Billing.log('purchaseOrderId place 1');
    var spinner,
        order_id = Billing.purchaseOrderId(aSku),
        story_id = DH.storage.readNumber('ST.promotingStoryId', 0);
    Billing.log('order_id', order_id, 'story_id', story_id);

    switch (aSku) {
    case 'story_write':
        console.log('story_write is purchased');
        break;
    case 'story_promote':
        // Promote story to the top
        spinner = DH.spinner(10);
        if (story_id <= 0) {
            console.warn('Undefined story_id during onPurchase, probably replayed promote after reinstall, ignored');
            Billing.markAsHandled(aSku);
            break;
        }
        // write purchase to server (in case user wants a refund)
        ST.request(
            'billing/promote-write.php',
            'story=' + encodeURIComponent(story_id) + '&order_id=' + encodeURIComponent(order_id),
            function (aData) {
                Billing.log('promote-write', aData);
                Billing.log('promote-write.message', aData.message);
                if (aData.code === 0) {
                    var s = DH.splash(
                        'Promoted',
                        'OK',
                        '#afa',
                        aData.message || 'Story was successfully promoted to the top!'
                    );
                    s.bgClickDisable();
                    Billing.markAsHandled(aSku);
                }
                spinner.hide();
            }
        );
        break;
    case 'story_limit':
        // Unlimited sentences
        spinner = DH.spinner(10);
        // write purchase to server (in case user want a refund)
        ST.request(
            'billing/limit-write.php',
            'story=' + encodeURIComponent(story_id || 1) + '&order_id=' + encodeURIComponent(order_id),
            function (aData) {
                Billing.log('limit-write', aData);
                Billing.log('limit-write.message', aData.message);
                if (aData.code === 0) {
                    var s = DH.splash(
                        'Unlocked',
                        'OK',
                        '#afa',
                        aData.message || 'You can now write any amount of sentences'
                    );
                    s.bgClickDisable();
                    Billing.markAsHandled(aSku);
                }
                spinner.hide();
            }
        );
        break;
    default:
        Billing.error('Unsupported SKU ' + aSku);
    }
};

Billing.onStore = function () {
    // Extra data saved with each purchase
    return {
        version: 1
    };
};

Billing.onRestore = function (aData) {
    // Restore data from order id
    Billing.log('onRestore', aData);
};

ST.bt = function () {
    Billing.test(['story_limit', 'story_promote']);
};

window.addEventListener('DOMContentLoaded', function () {
    Billing.init('https://ghost.sk/billing/', function () {
        var available = Billing.isAvailable(), e;
        Billing.log('billing is ' + available);
        // in story.html show paid block
        e = document.getElementById('paid');
        if (e && available) {
            console.log('Showing paid features');
            e.style.display = 'block';
        }
        // in story.html hide limit button
        e = document.getElementById('story_limit');
        if (e && Billing.purchaseExists('story_limit')) {
            console.log('Hiding limit button');
            e.style.display = 'none';
        }
    });
});

