// Nice interface for BillingForWebview.java
"use strict";
// globals: window, document, DH, setTimeout, Android

var BillingWeirdBug = 'A';

var BillingStartAt = Date.now();

var Billing = window.Billing || {};

BillingWeirdBug += 'B';


(function () {
    BillingWeirdBug += 'a';

    // in case Billing bind fails (but still creates object, it happens)
    if (!Billing.internalInit) {
        BillingWeirdBug += 'b';
        console.warn("Billing.internalInit not found, creating dummy interface!");
        Billing.init = function () { return false; };
        Billing.isReal = function () { return false; };
        Billing.isAvailable = function () { return false; };
        Billing.isAvailableDialog = function () { alert('Billing not available!'); };
        Billing.purchaseExists = function (aSku) {
            var BillingCache = DH.storage.readObject('BillingCache', {}),
                BillingExists = DH.storage.readObject('BillingExists', {});
            return BillingCache.hasOwnProperty(aSku) || BillingExists.hasOwnProperty(aSku);
        };
        Billing.purchases = function () { return {}; };
        Billing.purchase = function () { return false; };
        Billing.purchaseOrderId = function () { return ''; };
        Billing.handlePendingPurchases = function () { return false; };
        Billing.consume = function () { return false; };
        Billing.consumeAndPurchase = function () { return false; };
        Billing.log = function () { return false; };
        Billing.error = function () { return false; };
        BillingWeirdBug += 'c';
        return;
    }

    BillingWeirdBug += 'd';
    try {
        var BillingInitCalled = false,
            BillingInitDone = false,
            BillingPurchaseCalling = false,
            BillingConsumeCalling = false,
            BillingServer = '',
            BillingErrors = 0,
            BillingHandled = DH.storage.readObject('BillingHandled', {}),
            BillingCache = DH.storage.readObject('BillingCache', {}),
            BillingExists = DH.storage.readObject('BillingExists', {});

        Billing.errorAsAlert = DH.storage.readBoolean('Billing.errorAsAlert', false);

        BillingWeirdBug += 'e';
        function flattenArguments(aArguments) {
            // Convert array of arguments to string for logging
            var i, a = [], s;
            try {
                for (i = 0; i < aArguments.length; i++) {
                    if (aArguments[i] === null) {
                        a.push('null');
                        continue;
                    }
                    if (aArguments[i] === undefined) {
                        a.push('undefined');
                        continue;
                    }
                    if (aArguments[i] === 0) {
                        a.push('0');
                        continue;
                    }
                    if (aArguments[i] === false) {
                        a.push('false');
                        continue;
                    }
                    s = aArguments[i].toString();
                    try {
                        if (Array.isArray(aArguments[i])) {
                            s = '[' + aArguments[i].join('; ') + ']';
                        } else if (typeof aArguments[i] === 'object') {
                            s = JSON.stringify(aArguments[i]);
                        }
                    } catch (e) {
                        s += '(' + e + ')';
                    }
                    a.push(s);
                }
            } catch (e) {
                return aArguments.toString();
            }
            return a.join(', ');
        }
        BillingWeirdBug += '1';

        //Billing._BillingHandled = BillingHandled;
        //Billing._BillingCache = BillingCache;
        //Billing._flattenArguments = flattenArguments;

        Billing.log = function () {
            // Add all arguments to log
            var s = flattenArguments(arguments);
            if (Billing.errorAsAlert && s.match(/(error|exception)/i)) {
                alert(s);
            }
            Billing.internalLog(s);
        };

        Billing.error = function () {
            // Add all arguments to log
            BillingErrors++;
            var s = flattenArguments(arguments);
            if (Billing.errorAsAlert) {
                alert(s);
            }
            try {
                Billing.internalError(s);
            } catch (ignore) {
            }
            console.error(s);
            // send error to server directly
            DH.json(
                BillingServer + 'log/',
                {
                    cookie: DH.cookie(),
                    sku: 'ERROR',
                    data: s + '\n\n' + Billing.internalLogData()
                }
            );
        };

        Billing.isReal = function () {
            // Return true if this is real phone, not chrome
            return Billing.internalIsReal();
        };

        Billing.isAvailable = function () {
            // Return true if billing is available and fully initialized
            // note: internalIsAvailable is returning java boolean value which will be set to true after first init, but when called from multiple webviews they may not yet been initialized
            // note: maybe internalIsAvailable is not necessary?
            return Billing.internalIsAvailable() && BillingInitDone;
        };
        BillingWeirdBug += '2';

        Billing.isAvailableDialog = function () {
            // Shows dialog about billing not yet ready
            var a = Billing.isAvailable();
            if (!a) {
                DH.splash('Billing', 'OK', '#ffff77', 'Billing is not yet available, try again later...', null, 'auto', 'auto').bg.style.zIndex = 30;
            }
            return a;
        };

        function storeOrderOnServer(aPackageName, aSku, aSkus, aOrderId, aData, aCallbackOkResponse) {
            // Store data on server
            if (!aOrderId) {
                throw "Cannot store, undefined order id";
            }
            Billing.log('storeOrderOnServer', aSku, aOrderId, aData);
            try {
                DH.json(
                    BillingServer + 'store/',
                    {
                        version: 1,
                        cookie: DH.cookie(),
                        order_id: aOrderId,
                        data: JSON.stringify(aData),
                        log: Billing.logData().join('\n'),
                        package_name: aPackageName,
                        sku: aSku,
                        skus: aSkus.join(',')
                    },
                    function (aOk, aResponse) {
                        Billing.internalLogClear();
                        Billing.log('storeOrderOnServer finished', aOk, aResponse);
                        if (!aOk) {
                            Billing.error(aResponse);
                            aCallbackOkResponse(false, aResponse);
                            return;
                        }
                        if (aResponse.code !== 0) {
                            Billing.log('skus was', aSkus);
                            Billing.error('storeOrderOnServer response code ' + aResponse.code + ' - ' + aResponse.message);
                            aCallbackOkResponse(false, aResponse);
                            return;
                        }
                        aCallbackOkResponse(true, aResponse);
                    }
                );
            } catch (e) {
                Billing.error('storeOrderOnServer exception ' + e);
            }
        }
        BillingWeirdBug += '3';

        function validateSku(aSku) {
            // Throws exception if aSku is invalid
            if (typeof aSku !== 'string') {
                throw "SKU " + aSku + " is not a string but " + typeof aSku;
            }
            if (aSku.trim() === '') {
                throw "Empty SKU!";
            }
            if (aSku.trim() !== aSku) {
                throw "SKU '" + aSku + "' has extra spaces in it!";
            }
            if (!aSku.match(/^[a-z0-9_\.]{1,143}$/)) {
                throw "SKU '" + aSku + "' is invalid (only a-z0-9_. are allowed, max length 143)!";
            }
            if (aSku.match(/^[_\.]+/)) {
                throw "SKU '" + aSku + "' cannot start with _ or .";
            }
        }

        function markAsHandled(aSku) {
            // Mark sku as fully handled (i.e. user purchased vb_100_diamonds and you added them to game and shown alert)
            validateSku(aSku);
            var p = Billing.purchases();
            BillingHandled[aSku] = p[aSku] || {};
            DH.storage.writeObject('BillingHandled', BillingHandled);
            try {
                Billing.log('Marked ' + aSku + ' as handled, order_id=', p[aSku] && p[aSku].orderId);
            } catch (e) {
                console.error(e);
            }
        }
        Billing.markAsHandled = markAsHandled;

        function handlePendingPurchases() {
            // Find unhandled iaps and pass them to onPurchase for final handling
            Billing.log('handlePendingPurchases');
            if (typeof Billing.onPurchase !== 'function') {
                throw "Billing.onPurchase callback must be set before Billing.init()";
            }
            if (!BillingInitDone) {
                throw "Billing init not finished!";
            }
            function receiveStoreResponse(aOk, aResponse) {
                Billing.log('handlePendingPurchases store response ok=' + aOk + ' code=' + (aResponse && aResponse.code) + ' message=' + (aResponse && aResponse.message));
            }
            try {
                var k, iap, iaps, unhandled = {}, r = 0, b;
                iaps = Billing.purchases();
                Billing.log('iaps', iaps);
                Billing.log('handled', BillingHandled);
                // go through all iaps and find ones that were non handled yet
                for (k in iaps) {
                    if (iaps.hasOwnProperty(k)) {
                        iap = iaps[k];
                        if (BillingHandled.hasOwnProperty(iap.sku) && BillingHandled[iap.sku].orderId === iap.orderId) {
                            Billing.log('already handled iap ' + iap.sku + ' ' + iap.orderId);
                            continue;
                        }
                        Billing.log('unhandled iap ' + iap.sku + ' ' + iap.orderId, iap);
                        unhandled[k] = JSON.parse(JSON.stringify(iap));
                        r++;
                        // store on server (because order may not been saved, probably wasn't)
                        console.log('calling storeOrderOnServer from hpp!', iap);
                        if (!iap.cached) { // cached have no orderId only sku
                            storeOrderOnServer(iap.packageName, iap.sku, Object.keys(iaps), iap.orderId, Billing.onStore(), receiveStoreResponse);
                        }
                        // call onPurchase callback
                        if (Billing.onPurchase) {
                            try {
                                b = Billing.onPurchase(k);
                                // only mark as handled if callback returned true
                                if (b) {
                                    markAsHandled(k);
                                }
                            } catch (e) {
                                Billing.error('Billing.onPurchase error: ' + e);
                            }
                        }
                    }
                }
                Billing.log('all iaps handled');
                // Call unhandled callback
                //Billing.purchasesCallback(unhandled, aShowConsumablesNotification);
            } catch (e) {
                Billing.error('handlePendingPurchases error ' + e);
            }
            return r;
        }
        Billing.handlePendingPurchases = handlePendingPurchases;

        BillingWeirdBug += '4';

        Billing.init = function (aServer, aCallback) {
            // Initialize billing, call aCallback when done
            Billing.log('Billing.init ' + aServer);
            if (BillingInitCalled) {
                throw "Billing.init can only be called once!";
            }
            BillingInitCalled = true;

            // check missing dependencies
            if (!DH) {
                throw "Billing is missing DH dependency";
            }
            if (!DH.storage) {
                throw "Billing is missing DH.storage dependency";
            }
            if (!DH.json) {
                throw "Billing is missing DH.json dependency";
            }
            if (!DH.cookie) {
                throw "Billing is missing DH.cookie dependency";
            }
            if (!DH.spinner) {
                throw "Billing is missing DH.spinner dependency";
            }
            if (!DH.splash) {
                throw "Billing is missing DH.splash dependency";
            }
            if (!DH.options2) {
                console.warn("Billing is missing DH.options2 dependency");
            }
            if (!DH.console) {
                console.warn("Billing is missing DH.console dependency");
            }

            if (aServer !== undefined) {
                if (!aServer.match(/^http/)) {
                    throw "Billing server should start with http or be undefined!";
                }
                if (aServer.substr(-1) !== '/') {
                    throw "Billing server should end with /";
                }
            }

            BillingServer = aServer;
            if (typeof Billing.onPurchase !== 'function') {
                throw "Billing.onPurchase callback must be set before Billing.init()";
            }
            if (aServer && (typeof Billing.onStore !== 'function')) {
                throw "Billing.onStore callback must be function and defined before Billing.init()";
            }
            if (aServer && (typeof Billing.onRestore !== 'function')) {
                throw "Billing.onRestore callback must be function and defined before Billing.init()";
            }
            if (aCallback && (typeof aCallback !== 'function')) {
                throw "Billing.init(aServer, aCallback) - aCallback is not a function";
            }
            Billing.internalInitCallback = function () {
                Billing.log('Billing.internalInitCallback() called, available=', Billing.isAvailable());
                BillingInitDone = true;
                handlePendingPurchases();
                if (aCallback) {
                    aCallback();
                }
            };
            try {
                Billing.internalInit();
            } catch (e) {
                Billing.error('Billing.internalInit() throwed ' + e);
            }
            // if previous store failed, send it now
            if (DH.storage.keyExists('Billing.store.pendingOrder')) {
                var package_name = DH.storage.readString('Billing.store.pendingPackageName', ''),
                    skus = DH.storage.readArray('Billing.store.pendingSkus', []),
                    sku = DH.storage.readString('Billing.store.pendingSku', ''),
                    order = DH.storage.readString('Billing.store.pendingOrder', ''),
                    data = DH.storage.readObject('Billing.store.pendingData', {});
                // send it again
                Billing.log('init resends store sku', sku, 'skus', skus, 'order', order, 'data', data, 'package_name', package_name);
                storeOrderOnServer(package_name, sku, skus, order, data, function (aOk) {
                    if (aOk) {
                        DH.storage.erase('Billing.store.pendingPackageName');
                        DH.storage.erase('Billing.store.pendingSkus');
                        DH.storage.erase('Billing.store.pendingSku');
                        DH.storage.erase('Billing.store.pendingOrder');
                        DH.storage.erase('Billing.store.pendingData');
                    }
                });
            }
        };
        BillingWeirdBug += '5';

        Billing.logData = function () {
            // Return billing log as array of strings
            return Billing.internalLogData().split('\n');
        };

        Billing.logStore = function (aSku) {
            // Store billing log on server, e.g. after error or even successful purchase
            DH.json(
                BillingServer + 'log/',
                {
                    cookie: DH.cookie(),
                    sku: aSku,
                    data: BillingErrors + 'e ' + Billing.internalLogData()
                },
                function (aOk, aResponse) {
                    BillingErrors = 0;
                    Billing.log('log sent', aOk, aResponse);
                    Billing.internalLogClear();
                }
            );
        };

        Billing.purchases = function () {
            // Return array of purchases
            if (!BillingInitDone) {
                throw "Billing init not finished!";
            }
            var t1 = Date.now(),
                json = JSON.parse(Billing.internalPurchases()),
                k,
                changed,
                t2 = Date.now();
            Billing.log('Listed purchases in ' + (t2 - t1) + 'ms');
            // append cached purchases
            for (k in BillingCache) {
                if (BillingCache.hasOwnProperty(k)) {
                    if (!json.hasOwnProperty(k)) {
                        json[k] = BillingCache[k];
                    }
                }
            }
            // add missing purchases to exists
            changed = false;
            for (k in json) {
                if (json.hasOwnProperty(k)) {
                    if (!BillingExists.hasOwnProperty(k)) {
                        Billing.log('Adding missing sku ' + k + ' to BillingExists');
                        BillingExists[k] = 1;
                        changed = true;
                    }
                }
            }
            if (changed) {
                DH.storage.writeObject('BillingExists', BillingExists);
            }
            // note: handle unhandled purchases here?
            return json;
        };
        BillingWeirdBug += '6';

        function store(aSku, aCallbackOkResponse) {
            // Store extra data on server after each purchase (e.g. what clothes, how much currency player have)
            validateSku(aSku);
            if (!BillingServer) {
                throw "Cannot store data, no server";
            }
            if (typeof Billing.onStore !== 'function') {
                throw "Billing.onStore is not a function";
            }
            if (aCallbackOkResponse && typeof aCallbackOkResponse !== 'function') {
                throw "Billing store callback is not a function";
            }
            // get data
            var data = Billing.onStore(), iaps, iap;
            if (typeof data !== 'object') {
                throw "Billing.onStore did not returned object but " + typeof data;
            }
            if (!data.hasOwnProperty('version')) {
                throw "Cannot store data, no version in data";
            }
            // find iap
            iaps = Billing.purchases();
            if (!iaps.hasOwnProperty(aSku)) {
                console.error("Cannot store data, " + aSku + " not found among purchases " + (typeof iaps));
                aCallbackOkResponse(true, {code: 0, message: "Stored"});
                return;
                //throw "Cannot store data, " + aSku + " not found among purchases " + (typeof iaps);
            }
            Billing.log('store sku', aSku, 'iaps', iaps);
            iap = iaps[aSku];
            // mark that we are trying to send data
            DH.storage.writeArray('Billing.store.pendingSkus', Object.keys(iaps));
            DH.storage.writeString('Billing.store.pendingPackageName', (iap && iap.packageName));
            DH.storage.writeString('Billing.store.pendingSku', aSku);
            DH.storage.writeString('Billing.store.pendingOrder', (iap && iap.orderId));
            DH.storage.writeObject('Billing.store.pendingData', data);
            // send data
            storeOrderOnServer((iap && iap.packageName), aSku, Object.keys(iaps), (iap && iap.orderId), data, function (aOk, aResponse) {
                if (aOk) {
                    // mark that order was sent correctly
                    DH.storage.erase('Billing.store.pendingPackageName');
                    DH.storage.erase('Billing.store.pendingSkus');
                    DH.storage.erase('Billing.store.pendingSku');
                    DH.storage.erase('Billing.store.pendingOrder');
                    DH.storage.erase('Billing.store.pendingData');
                }
                if (aCallbackOkResponse) {
                    aCallbackOkResponse(aOk, aResponse);
                }
            });
        }
        BillingWeirdBug += '7';

        Billing.purchase = function (aSku, aCallbackOkSku) {
            // Purchase sku, call aCallbackOkSku with result
            handlePendingPurchases();
            validateSku(aSku);
            if (!BillingInitDone) {
                throw "Billing.purchase() can only be called after Billing.init() is finished!";
            }
            if (aCallbackOkSku && (typeof aCallbackOkSku !== 'function')) {
                throw "Billing.purchase(aSku, aCallbackOkSku) - aCallbackOkSku is set but not a function";
            }
            if (BillingPurchaseCalling) {
                throw "Another Billing.purchase() in progress!";
            }

            // already purchased this sku?
            var p = Billing.purchases(), spinner;
            if (p.hasOwnProperty(aSku)) {
                Billing.error('Purchasing SKU that already exists ' + aSku);
                if (aCallbackOkSku) {
                    aCallbackOkSku(true, aSku);
                } else {
                    Billing.onPurchase(aSku);
                }
                return;
            }

            // make purchase
            BillingPurchaseCalling = false;
            spinner = DH.spinner(10);
            Billing.internalPurchaseCallback = function (aPurchased, aSku2) {
                var b;
                spinner.hide();
                BillingPurchaseCalling = false;
                Billing.log('Billing.internalPurchaseCallback received ' + aPurchased + ' ' + aSku2);
                // this should not happen
                if (aSku !== aSku2) {
                    Billing.error('Purchased ' + aSku + ' but received ' + aPurchased + ' for ' + aSku2);
                }
                // callback
                try {
                    if (aCallbackOkSku) {
                        b = aCallbackOkSku(aPurchased, aSku);
                    } else {
                        if (aPurchased) {
                            b = Billing.onPurchase(aSku);
                        }
                    }
                    // only automatically mark as handled if callback returned true
                    if (aPurchased && (b === true)) {
                        markAsHandled(aSku);
                    }
                } catch (e) {
                    Billing.error(e);
                }
                // store it on server
                if (aPurchased && BillingServer) {
                    store(aSku, function (aOk, aResponse) {
                        Billing.log('stored ' + aOk + ' ' + JSON.stringify(aResponse));
                    });
                } else {
                    // store log of canceled purchase
                    Billing.logStore('!' + aSku);
                }
                // add to BillingExists
                if (aPurchased) {
                    BillingExists[aSku] = 1;
                    DH.storage.writeObject('BillingExists', BillingExists);
                }
            };
            Billing.internalPurchase(aSku);
        };
        BillingWeirdBug += '8';

        Billing.consume = function (aSku, aCallbackOkSku, aShowErrorDialog) {
            // Consume sku, call aCallbackOkSku with sku and if it was consumed
            validateSku(aSku);
            if (!BillingInitDone) {
                throw "Billing init not finished!";
            }
            if (aCallbackOkSku && (typeof aCallbackOkSku !== 'function')) {
                throw "Billing.consume(aSku, aCallbackOkSku) - aCallbackOkSku is set but not a function";
            }
            if (BillingConsumeCalling) {
                throw "Another Billing.purchase() in progress!";
            }

            // not purchased - consumption will always succeed (with warning)
            var p = Billing.purchases(), spinner;
            if (!p.hasOwnProperty(aSku)) {
                Billing.log('Consume will return true on non-owned SKU ' + aSku);
                // callback
                if (aCallbackOkSku) {
                    aCallbackOkSku(true, aSku);
                } else {
                    if (Billing.onConsume) {
                        Billing.onConsume(aSku);
                    }
                }
                return;
            }

            // only cached - remove from cache
            if (BillingCache.hasOwnProperty(aSku)) {
                Billing.log('Removing cached SKU ' + aSku);
                // delete from cache
                delete BillingCache[aSku];
                DH.storage.writeObject('BillingCache', BillingCache);
                // delete from exists
                delete BillingExists[aSku];
                DH.storage.writeObject('BillingExists', BillingExists);
                // callback
                if (aCallbackOkSku) {
                    aCallbackOkSku(true, aSku);
                } else {
                    if (Billing.onConsume) {
                        Billing.onConsume(aSku);
                    }
                }
                return;
            }

            // real consume
            BillingConsumeCalling = true;
            spinner = DH.spinner(10);
            Billing.internalConsumeCallback = function (aConsumed, aSku2) {
                spinner.hide();
                BillingConsumeCalling = false;
                Billing.log('internalConsumeCallback received ' + aConsumed + ' ' + aSku2);
                // this should not happen
                if (aSku !== aSku2) {
                    Billing.error('Called consume on ' + aSku + ' but received ' + aSku2);
                }
                // error dialog
                if (!aConsumed && aShowErrorDialog) {
                    Billing.log('Consume failed, showing error dialog');
                    DH.splash('Billing error', 'OK', 'pink', 'Cannot consume previous purchase, make sure you have Internet connection!', console.log, '80vw', 'auto').bg.style.zIndex = 30;
                }
                // callback
                if (aCallbackOkSku) {
                    aCallbackOkSku(aConsumed, aSku);
                } else {
                    if (Billing.onConsume) {
                        Billing.onConsume(aSku);
                    }
                }
                // delete from exists
                if (aConsumed) {
                    delete BillingExists[aSku];
                    DH.storage.writeObject('BillingExists', BillingExists);
                }
            };
            Billing.internalConsume(aSku);
        };

        Billing.consumeAndPurchase = function (aSku, aCallbackOkSku) {
            // Consume sku if needed and purchase another one (e.g. ingame currency)
            // doesn't exist, purchase new
            if (!Billing.purchaseExists(aSku)) {
                Billing.purchase(aSku, aCallbackOkSku);
                return;
            }
            // exists, consume it first
            Billing.consume(aSku, function (aConsumed) {
                if (!aConsumed) {
                    //
                    Billing.log('consumeAndPurchase not consumed so not purchasing ' + aSku);
                    if (aCallbackOkSku) {
                        aCallbackOkSku(false, aSku);
                    }
                    return;
                }
                if (!aCallbackOkSku && Billing.onConsume) {
                    Billing.onConsume(aSku);
                }
                // purchase
                Billing.purchase(aSku, aCallbackOkSku);
            }, true);
        };

        BillingWeirdBug += '9';
        Billing.purchaseExists = function (aSku) {
            // Returns true if IAP is purchased
            validateSku(aSku);
            if (!BillingInitDone) {
                console.warn('Calling purchaseExists before init is done, using BillingExists[' + aSku + ']');
                // exists?
                if (BillingExists.hasOwnProperty(aSku)) {
                    return true;
                }
                // cache?
                if (BillingCache.hasOwnProperty(aSku)) {
                    return true;
                }
                return false;
                //throw "Billing init not finished when testing purchaseExists(" + aSku + ")!";
            }
            var p = Billing.purchases();
            return p.hasOwnProperty(aSku) || BillingCache.hasOwnProperty(aSku); // testing cache is no longer needed but it doesn't hurt a thing
        };
        BillingWeirdBug += 'A';

        Billing.purchaseOrderId = function (aSku) {
            // Returns order id of purchase or empty string if it doesn't exists
            validateSku(aSku);
            if (!BillingInitDone) {
                throw "BillingInitDone=false when calling purchaseOrderId(" + aSku + ")!";
            }
            var p = Billing.purchases();
            return p.hasOwnProperty(aSku) ? p[aSku].orderId : '';
        };

        BillingWeirdBug += 'B';
        Billing.restore = function (aOrderId, aCallbackData) {
            // ask server for previously stored data and pass them to callback or onRestore
            Billing.log('Billing.restore', aOrderId);
            if (!aOrderId) {
                throw "Billing.restore order id undefined";
            }
            if (aCallbackData && typeof aCallbackData !== 'function') {
                throw "Billing.restore callback is not a function";
            }
            DH.json(
                BillingServer + 'restore/',
                {
                    cookie: DH.cookie(),
                    order_id: aOrderId
                },
                function (aOk, aResponse) {
                    Billing.log('Billing.restore response', aOk, aResponse);
                    if (!aOk || (aResponse.code !== 0)) {
                        DH.splash('Error', 'Ok', 'pink', (aResponse && aResponse.message) || 'Cannot restore', Billing.log, '80vw', 'auto').bg.style.zIndex = 30;
                        return;
                    }
                    // clear cache
                    DH.storage.writeObject('BillingCache', {});
                    BillingCache = {};
                    //Billing._BillingCache = BillingCache;
                    // find real purchases
                    var p = Billing.purchases(), o = {}, i, data;
                    // cache IAPs (only if they are not among real)
                    Billing.log('in restore response purchases are ', Object.keys(p).join(', '));
                    Billing.log('response skus are', aResponse.detail.skus);
                    for (i = 0; i < aResponse.detail.skus.length; i++) {
                        // if it is in p (real purchases) no need to cache it
                        if (p.hasOwnProperty(aResponse.detail.skus[i])) {
                            Billing.log('real purchase will NOT be added to cache', aResponse.detail.skus[i]);
                            continue;
                        }
                        Billing.log('will be in cache', aResponse.detail.skus[i]);
                        o[aResponse.detail.skus[i]] = {
                            sku: aResponse.detail.skus[i],
                            cached: true
                        };
                    }
                    DH.storage.writeObject('BillingCache', o);
                    BillingCache = o;
                    //Billing._BillingCache = o;
                    // parse data
                    try {
                        data = JSON.parse(aResponse.detail.data);
                    } catch (e) {
                        Billing.error(e);
                        DH.splash('Cannot restore extra data', 'Ok', 'pink', 'Saved data has invalid format and cannot be restored, try order number from different purchase!', Billing.log, '80vw', 'auto').bg.style.zIndex = 30;
                        return;
                    }
                    Billing.log('data', data);
                    // pass data to callback
                    try {
                        if (aCallbackData) {
                            aCallbackData(data);
                        } else {
                            Billing.onRestore(data);
                        }
                    } catch (e) {
                        Billing.error('Restore callback exception: ' + e);
                    }
                    // reset handled so that they are reapplied again
                    DH.storage.writeObject('BillingHandled', {});
                    BillingHandled = {};
                    // handle cached iaps now
                    // no longer true comment: no need to call markAsHandled(iap) for non-cached because it was already called at the begining
                    //if (aResponse.detail.skus.length > 0) {
                    handlePendingPurchases();
                    //}
                    // store log
                    Billing.logStore('RESTORE ' + aOrderId);
                }
            );
        };

        Billing.restorePrompt = function (aWhatIsBeingRestored, aCallbackData) {
            // Restore consumables from server using order id (user will receive in email)
            var s, input = document.createElement('input');
            // Ask for order ID
            s = DH.splash('Restore previous purchases', ['Restore', 'Cancel'], 'pink', function (aElement) {
                var p = document.createElement('p');
                // note
                p.innerHTML = 'If you reinstalled your phone (or you have a new phone) and you want to restore <b class="what"></b>, please type in <b>Order number</b> from <b>most recent</b> purchase. It should be in your email.';
                p.getElementsByClassName('what')[0].textContent = aWhatIsBeingRestored;
                aElement.appendChild(p);
                // input
                input.placeholder = 'GPA.XXXX-XXXX-XXXX-XXXXX';
                input.style.display = 'block';
                input.style.width = '100%';
                input.style.boxSizing = 'border-box';
                aElement.appendChild(input);
            }, function (aButton) {
                if (aButton === 'Restore') {
                    if (!input.value) {
                        return;
                    }
                    if (input.value.length < 5) {
                        DH.splash('Invalid order id', 'OK', 'pink', 'Value "' + input.value + '" doesn\'t look like valid order id. It should start with GPA and have a bunch of numbers. Please type it exactly as it was in the email.');
                        return;
                    }
                    var spinner = DH.spinner(10);
                    try {
                        Billing.restore(input.value, aCallbackData);
                    } finally {
                        spinner.hide();
                    }
                }
            }, '80vw', 'auto');
            s.bg.style.zIndex = 30;
            s.bgClickDisable();
        };

        Billing.test = function (aSkus) {
            // Show dialog where you can test all purchases and all API calls
            var i;
            for (i = 0; i < aSkus.length; i++) {
                validateSku(aSkus[i]);
            }
            function alert(aMessage) {
                // Non-blocking alert so that I can test fast click on purchase
                setTimeout(function () {
                    var s = DH.splash('Result', ['Ok', 'Ok'], 'white', flattenArguments([aMessage]), null, '95vw', 'auto');
                    s.bg.style.zIndex = 1000;
                    s.buttonsArray[1].hidden = true; // splash with 2 buttons will not hide on click so that it can be copied
                    s.content.style.whiteSpace = 'pre';
                    s.content.style.maxHeight = '80vh';
                    s.content.style.overflowY = 'scroll';
                }, 500);
            }
            DH.options2(function (aOptions, aBg) {
                aBg.style.zIndex = 999;
                aBg.style.lineHeight = '100%';
                aBg.style.backgroundColor = '#9ef5b6';
                aBg.style.fontFamily = 'sans-serif';
                aOptions.h1('Billing test');
                var log, data, order, sku = '';
                aOptions.buttons('Action', ['isAvailable', 'isReal', 'purchases', 'handlePendingPurchases'], function (event) {
                    try {
                        var p;
                        Billing.log(event.target.textContent, sku);
                        switch (event.target.textContent) {
                        case "isAvailable":
                            alert(Billing.isAvailable());
                            break;
                        case "isReal":
                            alert(Billing.isReal());
                            break;
                        case "purchases":
                            p = Billing.purchases();
                            alert(Object.keys(p).join(', ') + '\n\n' + JSON.stringify(p, undefined, 4));
                            break;
                        case "handlePendingPurchases":
                            alert(handlePendingPurchases());
                            break;
                        }
                    } catch (e) {
                        alert('Exception: ' + e);
                    }
                });
                aOptions.h2('Purchase and consume');
                aOptions.select('SKU', '', aSkus, function (aValue) {
                    sku = aValue;
                });
                aOptions.buttons('Action', ['purchase(p,alert)', 'purchase(p,-)', 'consume', 'purchaseExists', 'consumeAndPurchase', 'markAsHandled', 'purchaseOrderId(p)'], function (event) {
                    try {
                        Billing.log(event.target.textContent, sku);
                        switch (event.target.textContent) {
                        case "purchase(p,alert)":
                            Billing.purchase(sku, function (aOk, aName) {
                                alert(aOk + ' ' + aName);
                            });
                            break;
                        case "purchase(p,-)":
                            Billing.purchase(sku);
                            break;
                        case "consume":
                            Billing.consume(sku, function (aOk, aName) {
                                alert(aOk + ' ' + aName);
                            });
                            break;
                        case "consumeAndPurchase":
                            Billing.consumeAndPurchase(sku, function (aOk, aName) {
                                alert(aOk + ' ' + aName);
                            });
                            break;
                        case "purchaseExists":
                            alert(sku + ' ' + Billing.purchaseExists(sku));
                            break;
                        case "markAsHandled":
                            alert(markAsHandled(sku));
                            break;
                        case "purchaseOrderId(p)":
                            alert(Billing.purchaseOrderId(sku));
                            break;
                        }
                    } catch (e) {
                        alert('Exception: ' + e);
                    }
                });
                aOptions.h2('Store and restore');
                order = aOptions.text('OrderId', '');
                order.placeholder = 'GPA.XXXX-XXXX-XXXX-XXXXX';
                data = aOptions.text('Data', '');
                aOptions.buttons('Action', ['store', 'restore', 'restorePrompt', 'onStore', 'get orderId from sku'], function (event) {
                    try {
                        var p;
                        switch (event.target.textContent) {
                        case "store":
                            store(sku, function (aOk, aResponse) {
                                alert(aOk + ' ' + JSON.stringify(aResponse, undefined, 4));
                            });
                            break;
                        case "restore":
                            Billing.restore(order.value, function (aData) {
                                alert(JSON.stringify(aData, undefined, 4));
                            });
                            break;
                        case "restorePrompt":
                            Billing.restorePrompt('diamonds and stuff', function (aData) {
                                alert(JSON.stringify(aData, undefined, 4));
                            });
                            break;
                        case "onStore":
                            data.value = JSON.stringify(Billing.onStore());
                            break;
                        case "get orderId from sku":
                            p = Billing.purchases();
                            order.value = p[sku].orderId;
                            break;
                        }
                    } catch (e) {
                        alert('Exception: ' + e);
                    }
                });
                aOptions.h2('Debugging');
                log = aOptions.text('Log', '');
                aOptions.buttons('', ['log', 'logData', 'logStore', "console", "purge", "storage", "errorAsAlert=1", "errorAsAlert=0", "internalLogClear"], function (event) {
                    try {
                        switch (event.target.textContent) {
                        case "log":
                            Billing.log(log.value);
                            log.value = '';
                            break;
                        case "logData":
                            alert(Billing.logData().join('\n'));
                            break;
                        case "logStore":
                            alert(Billing.logStore('test'));
                            break;
                        case "console":
                            DH.console.show(true).style.zIndex = 1002;
                            break;
                        case "purge":
                            if (confirm('Erase localStorage and restart?')) {
                                var b = DH.storage.readBoolean('Billing.errorAsAlert');
                                DH.storage.eraseAll();
                                DH.storage.writeBoolean('Billing.errorAsAlert', b);
                                document.location.reload();
                                if (!Android.isReal()) {
                                    DH.console.disable();
                                }
                            }
                            break;
                        case "storage":
                            alert(DH.storage.debug());
                            break;
                        case "errorAsAlert=1":
                            DH.storage.writeBoolean('Billing.errorAsAlert', true);
                            Billing.errorAsAlert = true;
                            break;
                        case "errorAsAlert=0":
                            DH.storage.writeBoolean('Billing.errorAsAlert', false);
                            Billing.errorAsAlert = false;
                            break;
                        case "internalLogClear":
                            Billing.internalLogClear();
                            break;
                        }
                    } catch (e) {
                        alert('Exception: ' + e);
                    }
                });
                aOptions.saveCancel();
                window.a = aOptions;
                window.bg = aBg;
            });
        };

        BillingWeirdBug += 'f';
    } catch (e) {
        console.error('BillingWeirdBug ' + e);
        BillingWeirdBug += 'g';
        Billing.error('Billing error ' + e);
        alert(e);
    }
    BillingWeirdBug += 'h';

    Billing.log('av', Billing.isAvailable());
}());

BillingWeirdBug += 'i';

// end of nice interface

