// Common settings
"use strict";
// globals: window, DH, document, Android

var ST = ST || {};

// default settings
ST.appVersion = 59;
ST.server = DH.storage.readString('ST.server', 'https://ghost.sk/story/');
ST.markAsRead = DH.storage.readBoolean('ST.markAsRead', true);
ST.pass = 'story-2-app-key';
ST.passMod = DH.storage.readString('ST.passMod', '');
ST.cookie = DH.cookie();
ST.captcha = false;

ST.mod = function () {
    // request moderator password
    var p = prompt('Set moderator password', '');
    if (p) {
        DH.storage.writeString('ST.passMod', p);
        Android.reload();
    }
};

ST.local = function () {
    // Set local server on mobile
    DH.storage.writeString('ST.server', 'http://192.168.0.73/story/');
    Android.reload();
};

ST.localPurge = function () {
    // purge and make local
    DH.storage.eraseAll();
    ST.local();
};

ST.purge = function () {
    // Clear storage
    DH.storage.eraseAll();
    Android.reload();
};

ST.errorAsAlert = false;

ST.error = function (aError) {
    // default error display function
    if (ST.onError) {
        ST.onError(aError);
    }
    if (ST.errorAsAlert && aError) {
        alert(aError);
        return;
    }
    var e = document.getElementById('error');
    if (!e) {
        e = document.createElement('div');
        e.id = 'error';
        e.className = 'error';
        document.body.appendChild(e);
    }
    if (aError) {
        if (aError.toString().match('previously banned')) {
            console.warn(aError);
        } else if (aError.toString().match('one new story per day')) {
            console.warn(aError);
        } else {
            console.error(aError);
        }
    }
    e.textContent = aError;
    if (aError) {
        if (e.scrollIntoViewIfNeeded) {
            e.scrollIntoViewIfNeeded();
        } else {
            e.scrollIntoView();
        }
    }
};

ST.comboColor = function (aCombo) {
    // set combo color by it's value
    switch (aCombo.value) {
    case "1":
        // shadow (new)
        aCombo.style.color = 'gray';
        break;
    case "2":
        // spam (only author will see)
        aCombo.style.color = 'orange';
        break;
    case "3":
        // public
        aCombo.style.color = 'green';
        break;
    case "4":
        // ban
        aCombo.style.color = 'red';
        break;
    default:
        // error
        aCombo.style.color = 'fuchsia';
    }
};

ST.combo = function (aValue, aDataStory, aDataSentence, aChangeCallback) {
    // create one combo for moderation
    var select, option;
    select = document.createElement('select');
    select.addEventListener('change', aChangeCallback);

    option = document.createElement('option');
    option.value = 1;
    option.textContent = 'shadow';
    option.style.color = 'black';
    select.appendChild(option);

    option = document.createElement('option');
    option.value = 2;
    option.textContent = 'spam';
    option.style.color = 'orange';
    select.appendChild(option);

    option = document.createElement('option');
    option.value = 3;
    option.textContent = 'public';
    option.style.color = 'green';
    select.appendChild(option);

    option = document.createElement('option');
    option.value = 4;
    option.textContent = 'ban';
    option.style.color = 'red';
    select.appendChild(option);

    select.value = parseInt(aValue, 10);
    ST.comboColor(select);
    select.dataStory = aDataStory;
    select.dataSentence = aDataSentence;
    return select;
};

ST.comboQuality = function (aValue, aDataStory, aChangeCallback) {
    // create one combo for moderation
    var select, option;
    select = document.createElement('select');

    function setColor(aCombo) {
        var colors = {
            '0': 'gray',
            '1': 'blue',
            '2': 'black',
            '3': 'red'
        };
        aCombo.style.color = colors[aCombo.value];
    }

    select.addEventListener('change', function (event) {
        setColor(event.target);
    });
    select.addEventListener('change', aChangeCallback);
    select.title = 'Story quality (A=good story, B=average, C=low quality, Q=undecided)';
    select.placeholder = 'Q';

    option = document.createElement('option');
    option.value = 0;
    option.textContent = 'Q';
    option.style.color = 'gray';
    select.appendChild(option);

    option = document.createElement('option');
    option.value = 1;
    option.textContent = 'A';
    option.style.color = 'blue';
    select.appendChild(option);

    option = document.createElement('option');
    option.value = 2;
    option.textContent = 'B';
    option.style.color = 'black';
    select.appendChild(option);

    option = document.createElement('option');
    option.value = 3;
    option.textContent = 'C';
    option.style.color = 'red';
    select.appendChild(option);

    select.dataStory = aDataStory;
    select.style.color = 'gray';
    select.value = aValue || 0;
    setColor(select);
    return select;
};

