// Universal application metrics logging
// require: assert, cookie, request, storage, storage2, storage3, type, Android
"use strict";
// globals: document, window, Android, performance, setTimeout

var DH = DH || {};

DH.metrics = (function () {
    var self = {}, cookie, cookie3, key, app, version, url, pending = 0, abvalue = 0;
    self.verbose = false;
    self.errorLoggerEnabled = false;
    self.sessionId = -Math.abs(Math.random());
    self.sent = 0;
    self.ad = null;

    function abUpdate(aCookie) {
        // calculate value for A/B testing
        var i;
        abvalue = 0;
        for (i = 0; i < aCookie.length; i++) {
            abvalue += aCookie.charCodeAt(i);
        }
    }

    function cb(aValue) {
        // storage3 callback
        cookie3 = aValue;
        abUpdate(aValue || '');
    }
    cookie = DH.cookie(cb);
    cookie = cookie || cookie3;
    if (cookie) {
        abUpdate(cookie || '');
    }

    self.ab = function (aModulo) {
        // return value for A/B testing
        return abvalue % (aModulo || 2);
    };

    self.log = function (aEvent, aData, aLevel, aStars, aDuration) {
        // Log application event and data

        // main sequence
        try {
            if (!aStars) {
                if (window.hasOwnProperty('Android')) {
                    if (Android.hasOwnProperty('mainSequence')) {
                        aStars = Android.mainSequence();
                    }
                }
            }
        } catch (ignore) {
        }

        // if not duration given use session id (to detect crashes during billing)
        if (!aDuration) {
            aDuration = self.sessionId;
        }

        try {
            if (!app) {
                console.warn("DH.metrics not initialized");
                return;
            }
            if (pending >= 10) {
                console.log('DH.metrics.skipped', app, version, aEvent, aData, cookie, aLevel, aStars, aDuration);
                return;
            }
            if (self.verbose) {
                console.log('DH.metrics.log', app, version, aEvent, aData, cookie, aLevel, aStars, aDuration);
            }
            pending++;

            if (window.hasOwnProperty('Android')) {
                if (Android.hasOwnProperty('isConnectedWifi')) {
                    if (!Android.isConnectedWifi()) {
                        console.log('no wifi, no metrics');
                        return;
                    }
                }
            }

            self.sent++;
            DH.request.post(url,
                'key=' + encodeURIComponent(key) +
                '&app=' + encodeURIComponent(app) +
                '&version=' + encodeURIComponent(version) +
                '&cookie=' + encodeURIComponent(cookie) +
                '&event=' + encodeURIComponent(aEvent) +
                '&data=' + encodeURIComponent(aData || '') +
                '&level=' + encodeURIComponent(aLevel || 0) +
                '&stars=' + encodeURIComponent(aStars || 0) +
                '&duration=' + encodeURIComponent(aDuration || 0) +
                '&perf=' + encodeURIComponent(performance.now() || 0) +
                '&local=' + encodeURIComponent((new Date()).toISOString()),
                function (aData2) {
                    pending--;
                    if (self.verbose) {
                        console.log('metrics', aData2);
                    }
                    // for metrics-init receive ads
                    if (aEvent === 'metrics-init') {
                        try {
                            self.ad = JSON.parse(aData2);
                        } catch (ignore) {
                        }
                    }
                }, null, false
                );
        } catch (e) {
            console.error('DH.metrics.log ' + e);
        }
    };

    self.init = function (aAppName, aAppVersion, aAppKey, aServer, aSkipFirstLog) {
        // Set application name, version and (optional) key and server
        app = aAppName;
        version = parseInt(aAppVersion, 10);
        key = aAppKey || 'default-app-key';
        url = aServer || 'https://ghost.sk/metrics/send.php';
        if (!aSkipFirstLog) {
            self.log('metrics-init');
        }
    };

    self.errorLogger = function (aMaxErrors) {
        // enable error logger
        self.errorLoggerEnabled = true;
        self.errorsSent = 0;
        window.addEventListener('error', function (event) {
            self.errorEvent = event;
            self.errorsSent++;
            if (self.errorsSent <= (aMaxErrors || 10)) {
                try {
                    var o, f;
                    try {
                        f = event.filename.split('/').slice(-1)[0];
                    } catch (ignore2) {
                        f = event.filename;
                    }
                    o = f + ':' + event.lineno + ':' + event.colno + ': ' + event.message;
                    self.log('error', o);
                } catch (ignore) {
                }
            }
        });
    };

    self.showAd = function () {
        // Show ad
        if (!self.ad || self.ad.version !== 1) {
            return;
        }
        var i, cand = [], div, install, close, cur;
        // find ads for other apps
        for (i = 0; i < self.ad.items.length; i++) {
            if (self.ad.items[i].app !== app) {
                cand.push(self.ad.items[i]);
            }
        }
        // pick random ad
        if (cand.length <= 0) {
            return;
        }
        cur = cand[Math.floor(cand.length * Math.random())];
        // prepare elements
        function hide() {
            div.parentElement.removeChild(div);
        }
        // div
        div = document.createElement('div');
        div.style.position = 'fixed';
        div.style.left = '0';
        div.style.top = '0';
        div.style.width = '100vw';
        div.style.height = '100vh';
        div.style.zIndex = '9999999';
        div.style.backgroundColor = cur.backgroundColor || 'black';
        if (window.innerWidth > window.innerHeight) {
            div.style.backgroundImage = 'url(' + cur.backgroundImageHorizontal + ')';
        } else {
            div.style.backgroundImage = 'url(' + cur.backgroundImageVertical + ')';
        }
        div.style.backgroundPosition = 'center';
        div.style.backgroundSize = 'contain';
        div.style.backgroundRepeat = 'no-repeat';
        // close button
        close = document.createElement('button');
        close.textContent = 'X';
        close.style.float = 'right';
        close.style.backgroundColor = 'transparent';
        close.style.border = 'none';
        close.style.outline = 'none';
        close.style.boxShadow = 'none';
        close.style.color = 'red';
        close.style.textShadow = 'none';
        close.style.transition = 'opacity 0.3s';
        close.onclick = function () {
            close.disabled = true;
            close.style.opacity = 0;
            hide();
        };
        div.appendChild(close);
        // install button
        install = document.createElement('button');
        install.textContent = 'Install';
        install.style.position = 'fixed';
        install.style.left = 'calc(50vw - 5em)';
        install.style.width = '10em';
        install.style.top = '80vh';
        install.style.color = cur.installColor || 'black';
        install.style.backgroundColor = cur.installBackgroundColor || 'lime';
        install.style.boxShadow = 'inset 1px 1px 0 0 white, 0 0 1ex black';
        install.style.transition = 'opacity 0.3s';
        install.onclick = function () {
            install.disabled = true;
            install.style.opacity = 0;
            hide();
            if (window.hasOwnProperty('Android') && cur.installPackage) {
                window.Android.internalPlayStore(cur.installPackage);
                return;
            }
            window.open(cur.installLink, '_blank');
        };
        div.appendChild(install);
        // show
        document.body.appendChild(div);
        return div;
    };

    return self;
}());
