// Favorite stories (stories where user contributed)
"use strict";
// globals: document, window, DH, Android

var ST = ST || {};

ST.favorite = (function () {
    // Favorite stories
    var self = {};
    self.seen = DH.storage.readObject('ST.favorite.seen', {});

    self.add = function (aStoryId, aStoryName, aSeenSentences) {
        // remember how many sentences have user seen
        self.seen[aStoryId] = {name: aStoryName, count: aSeenSentences};
        DH.storage.writeObject('ST.favorite.seen', self.seen);
    };

    self.remove = function (aStoryId) {
        // remove from favorites (e.g. on dislike)
        if (self.seen.hasOwnProperty(aStoryId)) {
            delete self.seen[aStoryId];
            DH.storage.writeObject('ST.favorite.seen', self.seen);
        }
    };

    self.view = function (aStoryId, aStoryName, aSeenSentences) {
        // remember how many sentences have user seen but only when it was seen
        if (self.seen.hasOwnProperty(aStoryId)) {
            self.add(aStoryId, aStoryName, aSeenSentences);
        }
    };

    self.showNewCount = function () {
        // Detect number of new sentences
        if (Object.keys(self.seen).length === 0) {
            document.getElementById('envelope').textContent = '';
            return;
        }
        ST.request('favorites.php', 'stories=' + encodeURIComponent(Object.keys(self.seen).join(',')), function (aData) {
            console.log('count', self.seen, aData.detail);
            var fresh = 0, s, n, envelope = document.getElementById('envelope');
            for (s in self.seen) {
                if (self.seen.hasOwnProperty(s)) {
                    n = aData.detail.hasOwnProperty(s) ? aData.detail[s].count : self.seen[s].count;
                    fresh += n - self.seen[s].count;
                }
            }
            if (fresh > 0) {
                envelope.textContent = fresh + '📄';
                envelope.className = 'unread';
            } else {
                envelope.textContent = '📄';
                envelope.className = '';
            }
            console.log('fresh', fresh);
        });
    };

    function onFavorite(event) {
        // Click on favorite will open it
        console.log('favorite', event.target.dataStory);
        DH.storage.writeBoolean('ST.story.scroll', false);
        DH.storage.writeNumber('ST.story.code', parseInt(event.target.dataStory, 10));
        Android.loadUrl('file:///android_asset/story.html');
    }

    function onBack() {
        document.getElementById('stories').style.display = 'block';
        document.getElementById('start_new_story').style.display = 'block';
        document.getElementById('title').textContent = 'Story';
        document.getElementById('back').style.display = 'none';
        document.getElementById('favorites').style.display = 'none';
        document.getElementById('envelope').style.display = 'block';
    }

    self.showNewStories = function () {
        var parent = document.getElementById('favorites');
        parent.textContent = '';
        if (Object.keys(self.seen).length === 0) {
            return;
        }
        document.getElementById('envelope').style.display = 'none';
        document.getElementById('stories').style.display = 'none';
        document.getElementById('start_new_story').style.display = 'none';
        document.getElementById('title').textContent = 'Favorite stories';
        document.getElementById('back').style.float = 'left';
        document.getElementById('back').style.display = 'inline-block';
        document.getElementById('back').onclick = onBack;

        ST.request('favorites.php', 'stories=' + encodeURIComponent(Object.keys(self.seen).join(',')), function (aData) {
            console.log(aData);
            var s, dif, n, div, name, count;
            for (s in self.seen) {
                if (self.seen.hasOwnProperty(s)) {
                    if (!aData.detail || !aData.detail[s]) {
                        continue;
                    }
                    n = aData.detail.hasOwnProperty(s) ? aData.detail[s].count : self.seen[s].count;
                    dif = n - self.seen[s].count;
                    console.log('s', s, 'n', n, 'dif', dif);

                    div = document.createElement('div');
                    div.className = 'item';
                    div.dataStory = s;
                    div.onclick = onFavorite;

                    name = document.createElement('div');
                    name.className = 'name';
                    name.textContent = aData.detail[s].name;
                    name.dataStory = s;
                    div.appendChild(name);

                    if (dif > 0) {
                        count = document.createElement('div');
                        count.className = 'count';
                        count.textContent = dif + '📄';
                        count.dataStory = s;
                        div.appendChild(count);
                    }

                    parent.appendChild(div);
                }
            }
        });
    };

    return self;
}());
