// Standard response parser
"use strict";
// globals: window, document, DH, setTimeout, Android, Billing

var ST = ST || {};

ST.lastRequestEntire = '';

ST.response = function (aData) {
    // standard response parser
    var d;

    // clear previous error if any
    ST.error('');

    // no data
    if (!aData) {
        console.error('ND:' + ST.lastRequestEntire);
        ST.error('You must be online to read and write stories, or server is busy, try again later!');
        return;
    }

    // parse json
    try {
        window.d = aData;
        d = JSON.parse(aData);
    } catch (e) {
        console.error('Invalid response!', e, aData);
        ST.error('Server error, try again later!');
        return;
    }

    // unusual format
    if ((typeof d !== 'object') || (typeof d.code !== 'number') || (typeof d.message !== 'string')) {
        console.error('Unusual response!', d, aData);
        ST.error('Server error, try again later!');
        return;
    }

    // forbidden means update your app because app key has changed
    if ((d.code === -1) && (d.message === 'Forbidden')) {
        console.error('Forbidden', d, aData);
        if (document.location.pathname.match('moderate.html')) {
            ST.error('Your moderator password is incorrect!');
            setTimeout(ST.mod, 1000);
        } else {
            ST.error('Please update your app to latest version!');
        }
        return;
    }

    // negative response code (0=ok, >0=story id)
    if (d.code < 0) {
        if (aData.toString().match('You can start only one new story per day')) {
            console.warn('Warning', d, aData);
        } else {
            console.error('Error', d, aData);
        }
        ST.error(d.message);
        return;
    }

    return d;
};

ST.pagesForMods = ['moderate.php', 'unmoderated.php', 'mod-story-list.php', 'mod-story-rename.php', 'mod-story-status.php', 'mod-story-promote.php', 'mod-sentence-list.php', 'mod-sentence-rename.php', 'mod-author-recent.php', 'mod-story-quality.php'];

ST.request = function (aPage, aParams, aCallbackSuccess, aModPass) {
    // sandard request, then parser standard response and pass it to callback
    console.log('ST.request', aPage, aParams);
    var pass = ST.pagesForMods.indexOf(aPage) >= 0 ? ST.passMod : ST.pass, cached_stories, cached_stories_time, res2;
    if (aModPass && ST.passMod) {
        pass = ST.passMod;
    }

    ST.iap_story_limit = '';
    if (window.hasOwnProperty('Billing') && typeof Billing === "object" && Billing.log) {
        Billing.log('purchaseOrderId place 2');
        if (Billing && Billing.isAvailable && Billing.isAvailable()) {
            Billing.log('place 3');
            ST.iap_story_limit = Billing.purchaseOrderId('story_limit');
        }
        if (Billing && Billing.isAvailable && !Billing.isAvailable()) {
            if (Billing.purchaseExists('story_limit')) {
                ST.iap_story_limit = 'GPA.waiting';
            }
        }
    }
    console.log('ST.iap_story_limit', ST.iap_story_limit);

    ST.lastRequestEntire = ST.server + aPage + '?' + 'pass=' + encodeURIComponent(pass) +
        '&flushCache=' + encodeURIComponent(Date.now()) +
        '&cookie=' + encodeURIComponent(ST.cookie) +
        '&iap_story_limit=' + encodeURIComponent(ST.iap_story_limit) +
        '&' + aParams + '&zzz=1';

    if (aPage === 'stories.php') {
        cached_stories = DH.storage.readString('ST.cache.stories', '');
        cached_stories_time = DH.storage.readNumber('ST.cache.storiesTime', 0);
        if (cached_stories && (Date.now() - cached_stories_time < 30000)) {
            console.log('cached stories', Date.now() - cached_stories_time, 'data', cached_stories.substr(0, 150) + '...');
            res2 = ST.response(cached_stories, 'error');
            if (res2) {
                aCallbackSuccess(res2);
            }
            return;
        }
    }

    DH.request.post(ST.server + aPage + '?cache=' + Date.now().toString(),
        'pass=' + encodeURIComponent(pass) +
        '&flushCache=' + encodeURIComponent(Date.now()) +
        '&cookie=' + encodeURIComponent(ST.cookie) +
        '&iap_story_limit=' + encodeURIComponent(ST.iap_story_limit) +
        '&' + aParams,
        function (aData) {
            // parse response
            ST.lastPage = aPage;
            ST.lastParams = aParams;
            ST.lastCookie = ST.cookie;
            ST.lastPass = pass;
            ST.lastData = aData;
            var res = ST.response(aData, 'error');
            if (res) {
                try {
                    if (res.code === 0 && aPage === 'stories.php' && aData  && (aData.length > 1000)) {
                        console.log('saving stories to cache');
                        DH.storage.writeString('ST.cache.stories', aData);
                        DH.storage.writeNumber('ST.cache.storiesTime', Date.now());
                    }
                } catch (e) {
                    console.error(e);
                }
                aCallbackSuccess(res);
            }
        }
        );
};

ST.requestMod = function (aPage, aParams, aCallbackSuccess) {
    ST.request(aPage, aParams, aCallbackSuccess, true);
};

ST.silentRequest = function (aPage, aParams, aCallback, aModPass) {
    // Make silent request (response is not parsed and errors are not displayed)
    console.log('ST.silentRequest', aPage, aParams);
    var pass = ST.pagesForMods.indexOf(aPage) >= 0 ? ST.passMod : ST.pass;
    if (aModPass && ST.passMod) {
        pass = ST.passMod;
    }
    DH.request.post(ST.server + aPage,
        'pass=' + encodeURIComponent(pass) +
        '&flushCache=' + encodeURIComponent(Date.now()) +
        '&cookie=' + encodeURIComponent(ST.cookie) +
        '&' + aParams,
        function (aData) {
            // parse response
            ST.lastPage = aPage;
            ST.lastParams = aParams;
            ST.lastCookie = ST.cookie;
            ST.lastPass = pass;
            ST.lastData = aData;
            aCallback(aData);
        }
        );
};
