// Show story and let user add new sentence
"use strict";
// globals: DH, window, document, Android, setTimeout, Billing

var ST = ST || {};
ST.story = { "code": 1 };

ST.unseen = function (aStoryId, aSentenceIndex) {
    // return true if this story id was not previously seen, remember this sentence as seen
    var key = 'ST_' + aStoryId,
        last = DH.storage.readNumber(key, -1),
        uns = aSentenceIndex > last;
    if (uns && ST.markAsRead) {
        DH.storage.writeNumber(key, aSentenceIndex);
    }
    return uns;
};

ST.renderStory = function (aStoryId, aNewMessage, aCallback) {
    // render one story
    var cap = document.getElementById('captcha_text') ? document.getElementById('captcha_text').value : '';
    console.log(ST.server + 'story.php?story=' + encodeURIComponent(aStoryId) + (aNewMessage ? '&message=' + encodeURIComponent(aNewMessage) : ''));
    ST.request('story.php', 'story=' + encodeURIComponent(aStoryId) + (aNewMessage ? '&message=' + encodeURIComponent(aNewMessage) : '') + '&captcha=' + encodeURIComponent(cap),
        function (aData) {
            var i, content = document.getElementById('content'), span;
            ST.story = aData;
            console.log('Story #' + ST.story.code + ' named "' + ST.story.message + '" has ' + ST.story.detail.length + ' sentences');

            // render heading
            document.getElementById('title').textContent = ST.story.message;
            ST.story.name = ST.story.message;

            // render sentences
            content.innerText = '';
            for (i = 0; i < ST.story.detail.length; i++) {

                // special sentences
                // line break
                if (ST.story.detail[i] === '#br') {
                    span = document.createElement('br');
                    content.appendChild(span);
                    continue;
                }
                // paragraph
                if (ST.story.detail[i] === '#p') {
                    span = document.createElement('br');
                    content.appendChild(span);
                    span = document.createElement('br');
                    content.appendChild(span);
                    continue;
                }
                // sentence that was left out due to rewrite
                if (ST.story.detail[i] === '#empty') {
                    continue;
                }

                // normal sentence
                span = document.createElement('span');
                span.className = 'sentence';
                if (ST.unseen(aStoryId, i)) {
                    span.classList.add('new');
                }
                span.textContent = ST.story.detail[i] + ' ';
                content.appendChild(span);
            }
            // make reply box visible now
            if (aStoryId !== 1) {
                ST.showReplyBox();
                // remaining sentences for author
                if (!Billing.purchaseExists('story_limit')) {
                    if (aData.hasOwnProperty('extra') && aData.extra.hasOwnProperty('remaining')) {
                        if (aData.extra.remaining === 2) {
                            document.getElementById('reply').placeholder = 'Two sentences remaining!';
                        }
                        if (aData.extra.remaining === 1) {
                            document.getElementById('reply').placeholder = 'One sentence remaining!';
                        }
                        if (aData.extra.remaining === 0) {
                            document.getElementById('reply').placeholder = 'You wrote last three sentences, let others write now!';
                            document.getElementById('reply').disabled = true;
                            document.getElementById('submit').style.display = 'none';
                            document.getElementById('captcha').style.display = 'none';
                        }
                    }
                }
            }
            // likes and dislikes
            if (aData.hasOwnProperty('extra')) {
                ST.vote.render(aStoryId, aData.extra.likes, aData.extra.dislikes);
            }
            // if empty focus textarea and show keyboard
            if (ST.story.detail.length <= 0) {
                console.log('Focusing reply');
                document.getElementById('reply').focus();
                Android.keyboardImplicit();
            }
            // callback when its done
            if (aCallback) {
                aCallback(aData);
            }
            // mark as seen
            ST.favorite.view(aStoryId, ST.story.name, ST.story.detail.length);
            // show paid features
            if (Billing.isAvailable()) {
                document.getElementById('paid').style.display = 'block';
            }
        });
};

ST.normalizeSentence = function (aSentence) {
    // normalize sentence (title case, dot at the end)
    // trim leading eol
    var s = aSentence.trim();
    if (s === '') {
        return '';
    }
    // leave known #commands as is
    if (['#br', '#p', '#empty'].indexOf(s) >= 0) {
        return s;
    }
    // Title case
    s = s.substr(0, 1).toUpperCase() + s.substr(1);
    // ending with dot (unless ?!)
    if (['?', '!', '.', '"'].indexOf(s.substr(-1)) < 0) {
        s = s + '.';
    }
    return s;
};

ST.showReplyBox = function () {
    // Show reply box after story was fully loaded and user have access
    document.getElementById('replybox').style.display = 'block';
    if (ST.captcha) {
        document.getElementById('captcha').style.display = 'flex';
        document.getElementById('captcha_image').src = 'http://localhost/story/captcha.php?cookie=' + encodeURIComponent(ST.cookie) + '&date=' + encodeURIComponent(Date.now());
        document.getElementById('captcha_text').value = '';
    }
};

ST.onReplyInput = function (event) {
    // show remaining characters
    document.getElementById('limit').textContent = (event.target.value.length + 1) + '/250';
    ST.active.stopMeasuringTime();
};

ST.onSubmitClick = function () {
    // submit new sentence to the story
    var s, sp;
    if (!window.hasOwnProperty('Billing') || !Billing.isAvailable()) {
        DH.splash('Error', 'OK', 'pink', 'Billing service is not available, wait please...');
        return;
    }
    if (!Billing.purchaseExists('story_limit')) {
        sp = DH.splash('Wait', ['Buy', 'Cancel'], 'skyblue', 'To became writer you need to make a one-time purchase of $1', function (aButton) {
            if (aButton === 'Buy') {
                Billing.purchase('story_limit', function (aOk, aSku) {
                    console.log('purchased', aOk, aSku);
                    if (aOk) {
                        ST.onSubmitClick();
                    }
                });
            }
        }, '80vw');
        sp.bgClickDisable();
        return;
    }


    s = ST.normalizeSentence(document.getElementById('reply').value);
    if (s === '') {
        return;
    }

    // disable submit button
    ST.onError = function () {
        document.getElementById('submit').textContent = 'Submit';
    };
    document.getElementById('submit').textContent = 'Wait…';
    document.getElementById('submit').disable = true;
    document.getElementById('reply').focus();

    // check max length (on android gboard will sometimes allow longer that maxlength, also dot may be added in normalizer)
    if (s.length > 250) {
        document.getElementById('submit').textContent = 'Submit';
        document.getElementById('limit').style.color = 'red';
        setTimeout(function () {
            document.getElementById('limit').style.color = '';
        }, 500);
        return;
    }

    // add and refresh
    ST.renderStory(ST.story.code, s, function (aData) {
        console.log('after submit', aData);
        document.getElementById('reply').value = '';
        document.getElementById('limit').textContent = '';
        document.getElementById('reply').focus();
        if (document.getElementById('bottom').scrollIntoViewIfNeeded) {
            document.getElementById('bottom').scrollIntoViewIfNeeded();
        } else {
            document.getElementById('bottom').scrollIntoView();
        }
        document.getElementById('submit').textContent = 'Submit';
        document.getElementById('submit').disable = false;
        // add this story to favorite
        ST.favorite.add(ST.story.code, ST.story.name, aData.detail.length);
    });
};

ST.onReplyKeyPress = function (event) {
    // enter will submit sentence
    if (event.key === 'Enter') {
        ST.onSubmitClick();
        event.preventDefault();
    }
};

ST.onBack = function () {
    // Go back
    ST.active.report(console.log);
    Android.loadUrl('file:///android_asset/android.html');
};

ST.onPromote = function () {
    // Promote this story
    if (!Billing.isAvailable()) {
        DH.splash('Oops', 'OK', 'pink', 'Billing is not yet available, try again later...');
        return;
    }
    var s = DH.spinner(10);
    DH.storage.writeNumber('ST.promotingStoryId', ST.story.code);
    ST.request('billing/promote-begin.php', 'story=' + ST.story.code, function (aData) {
        console.log(aData);
        s.hide();
        if (aData.detail.available) {
            // promote available
            DH.splash(
                'Promote story',
                ['Promote', 'No thanks'],
                '#afa',
                function (aParent) {
                    aParent.textContent = aData.detail.message;
                },
                function (aButton) {
                    if (aButton === 'Promote') {
                        Billing.consumeAndPurchase('story_promote');
                    }
                },
                '85%',
                'auto'
            ).bgClickDisable();
        } else {
            // promote not available
            DH.splash(
                'Oops!',
                ['Close'],
                'pink',
                function (aParent) {
                    aParent.textContent = aData.detail.message;
                },
                null,
                '85%',
                'auto'
            );
        }
    });
};

ST.onStoryLimit = function () {
    // Unlimited sentences
    if (!Billing.isAvailable()) {
        DH.splash('Oops', 'OK', 'pink', 'Billing is not yet available, try again later...');
        return;
    }
    var s = DH.spinner(10);
    ST.request('billing/limit-begin.php', 'story=' + ST.story.code, function (aData) {
        console.log('response data', aData);
        s.hide();
        if (aData.detail.available) {
            // iap available
            DH.splash(
                'Unlimited sentences',
                ['Unlock', 'No thanks'],
                '#afa',
                function (aParent) {
                    aParent.textContent = aData.detail.message;
                },
                function (aButton) {
                    if (aButton === 'Unlock') {
                        Billing.consumeAndPurchase('story_limit');
                    }
                },
                '85%',
                'auto'
            ).bgClickDisable();
        } else {
            // iap not available
            DH.splash(
                'Oops!',
                ['Close'],
                'pink',
                function (aParent) {
                    aParent.textContent = aData.detail.message;
                },
                null,
                '85%',
                'auto'
            );
        }
    });
};

window.addEventListener('DOMContentLoaded', function () {
    // initialize window
    DH.console.showOnMultipleClicks(true);
    // metrics
    DH.metrics.init('story', ST.appVersion || 0, undefined, undefined, true);
    DH.metrics.errorLogger(10);

    // get the story id
    ST.story.code = DH.storage.readNumber('ST.story.code', 0);
    console.log('ST.story.code', ST.story.code);
    //DH.metrics.log('story', ST.story.code);

    // render story
    var x = DH.storage.readNumber('ST_' + ST.story.code), java;
    ST.renderStory(ST.story.code, '', function () {
        var n = document.getElementsByClassName('new');
        if (n && (n.length > 0)) {
            if (n[0].scrollIntoViewIfNeeded) {
                n[0].scrollIntoViewIfNeeded();
            } else {
                n[0].scrollIntoView();
            }
        } else {
            // if user seen this story and nothing is new scroll to the bottom so that user can write
            if (x > 0) {
                if (document.getElementById('bottom').scrollIntoViewIfNeeded) {
                    document.getElementById('bottom').scrollIntoViewIfNeeded();
                } else {
                    document.getElementById('bottom').scrollIntoView();
                }
            }
        }
    });

    // callbacks
    document.getElementById('reply').addEventListener('input', ST.onReplyInput);
    document.getElementById('reply').addEventListener('keypress', ST.onReplyKeyPress, true);
    document.getElementById('submit').addEventListener('click', ST.onSubmitClick);
    document.getElementById('back').addEventListener('click', ST.onBack);

    // focus to first unseen sentence
    if (document.getElementById('bottom').scrollIntoViewIfNeeded) {
        document.getElementById('bottom').scrollIntoViewIfNeeded();
    } else {
        document.getElementById('bottom').scrollIntoView();
    }

    // set back to return to current story
    java = 'java';
    Android.setBack(java + 'script:ST.onBack()');

    // promote
    document.getElementById('promote').addEventListener('click', ST.onPromote);
    if (ST.story.code === 1) {
        document.getElementById('promote').style.display = 'none';
    }

    // story_limit
    document.getElementById('story_limit').addEventListener('click', ST.onStoryLimit);
    if ((ST.story.code === 1) || ST.iap_story_limit) {
        document.getElementById('story_limit').style.display = 'none';
    }
});
